@@ -38,11 +38,36 @@ run. If you're passing arguments to both Cargo and the binary, the ones after
 pub fn exec(config: &mut Config, args: &ArgMatches) -> CliResult {
     let ws = args.workspace(config)?;
 
-    let mut compile_opts = args.compile_options_for_single_package(config, CompileMode::Build)?;
+    let mut compile_opts = args.compile_options(config, CompileMode::Build)?;
     if !args.is_present("example") && !args.is_present("bin") {
-        compile_opts.filter = CompileFilter::Default {
-            required_features_filterable: false,
-        };
+        let default_runs: Vec<_> = compile_opts
+            .spec
+            .get_packages(&ws)?
+            .iter()
+            .filter_map(|pkg| pkg.manifest().default_run())
+            .collect();
+        if default_runs.len() == 1 {
+            compile_opts.filter = CompileFilter::new(
+                false,
+                vec![default_runs[0].to_owned()],
+                false,
+                vec![],
+                false,
+                vec![],
+                false,
+                vec![],
+                false,
+                false,
+            );
+        } else {
+            // ops::run will take care of errors if len pkgs != 1.
+            compile_opts.filter = CompileFilter::Default {
+                // Force this to false because the code in ops::run is not
+                // able to pre-check features before compilation starts to
+                // enforce that only 1 binary is built.
+                required_features_filterable: false,
+            };
+        }
     };
     match ops::run(&ws, &compile_opts, &values(args, "args"))? {
         None => Ok(()),
