@@ -35,44 +35,57 @@
 //! then you use `chain_err` to tack on more context for why the feature was
 //! required when the feature isn't activated.
 //!
+//! 4. Update the unstable documentation at
+//!    `src/doc/src/reference/unstable.md` to include a short description of
+//!    how to use your new feature.  When the feature is stabilized, be sure
+//!    that the Cargo Guide or Reference is updated to fully document the
+//!    feature and remove the entry from the Unstable section.
+//!
 //! And hopefully that's it! Bear with us though that this is, at the time of
 //! this writing, a very new feature in Cargo. If the process differs from this
 //! we'll be sure to update this documentation!
 
+use std::cell::Cell;
 use std::env;
 use std::fmt;
 use std::str::FromStr;
 
+use failure::Error;
+
 use util::errors::CargoResult;
 
-/// The epoch of the compiler (RFC 2052)
+/// The edition of the compiler (RFC 2052)
 #[derive(Clone, Copy, Debug, Hash, PartialOrd, Ord, Eq, PartialEq, Serialize, Deserialize)]
-pub enum Epoch {
-    /// The 2015 epoch
-    Epoch2015,
-    /// The 2018 epoch
-    Epoch2018,
+pub enum Edition {
+    /// The 2015 edition
+    Edition2015,
+    /// The 2018 edition
+    Edition2018,
 }
 
-impl fmt::Display for Epoch {
+impl fmt::Display for Edition {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
-            Epoch::Epoch2015 => f.write_str("2015"),
-            Epoch::Epoch2018 => f.write_str("2018"),
+            Edition::Edition2015 => f.write_str("2015"),
+            Edition::Edition2018 => f.write_str("2018"),
         }
     }
 }
-impl FromStr for Epoch {
-    type Err = ();
-    fn from_str(s: &str) -> Result<Self, ()> {
+impl FromStr for Edition {
+    type Err = Error;
+    fn from_str(s: &str) -> Result<Self, Error> {
         match s {
-            "2015" => Ok(Epoch::Epoch2015),
-            "2018" => Ok(Epoch::Epoch2018),
-            _ => Err(()),
+            "2015" => Ok(Edition::Edition2015),
+            "2018" => Ok(Edition::Edition2018),
+            s => {
+                bail!("supported edition values are `2015` or `2018`, but `{}` \
+                       is unknown", s)
+            }
         }
     }
 }
 
+#[derive(PartialEq)]
 enum Status {
     Stable,
     Unstable,
@@ -94,7 +107,7 @@ macro_rules! features {
             $(
                 pub fn $feature() -> &'static Feature {
                     fn get(features: &Features) -> bool {
-                        features.$feature
+                        stab!($stab) == Status::Stable || features.$feature
                     }
                     static FEAT: Feature = Feature {
                         name: stringify!($feature),
@@ -127,8 +140,12 @@ macro_rules! features {
 }
 
 macro_rules! stab {
-    (stable) => (Status::Stable);
-    (unstable) => (Status::Unstable);
+    (stable) => {
+        Status::Stable
+    };
+    (unstable) => {
+        Status::Unstable
+    };
 }
 
 /// A listing of all features in Cargo
@@ -156,14 +173,26 @@ features! {
         // Downloading packages from alternative registry indexes.
         [unstable] alternative_registries: bool,
 
-        // Using epochs
-        [unstable] epoch: bool,
+        // Using editions
+        [stable] edition: bool,
 
         // Renaming a package in the manifest via the `package` key
         [unstable] rename_dependency: bool,
 
         // Whether a lock file is published with this crate
         [unstable] publish_lockfile: bool,
+
+        // Overriding profiles for dependencies.
+        [unstable] profile_overrides: bool,
+
+        // Separating the namespaces for features and dependencies
+        [unstable] namespaced_features: bool,
+
+        // "default-run" manifest option,
+        [unstable] default_run: bool,
+
+        // Declarative build scripts.
+        [unstable] metabuild: bool,
     }
 }
 
@@ -286,6 +315,9 @@ pub struct CliUnstable {
     pub no_index_update: bool,
     pub avoid_dev_deps: bool,
     pub minimal_versions: bool,
+    pub package_features: bool,
+    pub advanced_env: bool,
+    pub config_profile: bool,
 }
 
 impl CliUnstable {
@@ -319,6 +351,9 @@ impl CliUnstable {
             "no-index-update" => self.no_index_update = true,
             "avoid-dev-deps" => self.avoid_dev_deps = true,
             "minimal-versions" => self.minimal_versions = true,
+            "package-features" => self.package_features = true,
+            "advanced-env" => self.advanced_env = true,
+            "config-profile" => self.config_profile = true,
             _ => bail!("unknown `-Z` flag specified: {}", k),
         }
 
@@ -327,17 +362,59 @@ impl CliUnstable {
 }
 
 fn channel() -> String {
-    env::var("__CARGO_TEST_CHANNEL_OVERRIDE_DO_NOT_USE_THIS").unwrap_or_else(|_| {
-        ::version()
-            .cfg_info
-            .map(|c| c.release_channel)
-            .unwrap_or_else(|| String::from("dev"))
-    })
+    if let Ok(override_channel) = env::var("__CARGO_TEST_CHANNEL_OVERRIDE_DO_NOT_USE_THIS") {
+        return override_channel;
+    }
+    if let Ok(staging) = env::var("RUSTC_BOOTSTRAP") {
+        if staging == "1" {
+            return "dev".to_string();
+        }
+    }
+    ::version()
+        .cfg_info
+        .map(|c| c.release_channel)
+        .unwrap_or_else(|| String::from("dev"))
 }
 
-fn nightly_features_allowed() -> bool {
-    match &channel()[..] {
-        "nightly" | "dev" => true,
+thread_local!(
+    static NIGHTLY_FEATURES_ALLOWED: Cell<bool> = Cell::new(false);
+    static ENABLE_NIGHTLY_FEATURES: Cell<bool> = Cell::new(false);
+);
+
+/// This is a little complicated.
+/// This should return false if:
+/// - this is an artifact of the rustc distribution process for "stable" or for "beta"
+/// - this is an `#[test]` that does not opt in with `enable_nightly_features`
+/// - this is a integration test that uses `ProcessBuilder`
+///      that does not opt in with `masquerade_as_nightly_cargo`
+/// This should return true if:
+/// - this is an artifact of the rustc distribution process for "nightly"
+/// - this is being used in the rustc distribution process internally
+/// - this is a cargo executable that was built from source
+/// - this is an `#[test]` that called `enable_nightly_features`
+/// - this is a integration test that uses `ProcessBuilder`
+///       that called `masquerade_as_nightly_cargo`
+pub fn nightly_features_allowed() -> bool {
+    if ENABLE_NIGHTLY_FEATURES.with(|c| c.get()) {
+        return true
+    }
+     match &channel()[..] {
+        "nightly" | "dev" => NIGHTLY_FEATURES_ALLOWED.with(|c| c.get()),
         _ => false,
     }
 }
+
+/// Allows nightly features to be enabled for this thread, but only if the
+/// development channel is nightly or dev.
+///
+/// Used by cargo main to ensure that a cargo build from source has nightly features
+pub fn maybe_allow_nightly_features() {
+    NIGHTLY_FEATURES_ALLOWED.with(|c| c.set(true));
+}
+
+/// Forcibly enables nightly features for this thread.
+///
+/// Used by tests to allow the use of nightly features.
+pub fn enable_nightly_features() {
+    ENABLE_NIGHTLY_FEATURES.with(|c| c.set(true));
+}
