@@ -161,9 +161,9 @@ fn run_doc_tests(
             deps,
         } = doctest_info;
         config.shell().status("Doc-tests", target.name())?;
-        let mut p = compilation.rustdoc_process(package)?;
+        let mut p = compilation.rustdoc_process(package, target)?;
         p.arg("--test")
-            .arg(target.src_path())
+            .arg(target.src_path().path())
             .arg("--crate-name")
             .arg(&target.crate_name());
 
@@ -193,8 +193,8 @@ fn run_doc_tests(
             }
         }
 
-        for &(ref target, ref lib) in deps.iter() {
-            let mut arg = OsString::from(target.crate_name());
+        for &(ref extern_crate_name, ref lib) in deps.iter() {
+            let mut arg = OsString::from(extern_crate_name);
             arg.push("=");
             arg.push(lib);
             p.arg("--extern").arg(&arg);
