@@ -87,9 +87,9 @@ pub fn without_prefix<'a>(long_path: &'a Path, prefix: &'a Path) -> Option<&'a P
 
 pub fn resolve_executable(exec: &Path) -> CargoResult<PathBuf> {
     if exec.components().count() == 1 {
-        let paths = env::var_os("PATH").ok_or(format_err!("no PATH"))?;
+        let paths = env::var_os("PATH").ok_or_else(|| format_err!("no PATH"))?;
         let candidates = env::split_paths(&paths).flat_map(|path| {
-            let candidate = PathBuf::from(path).join(&exec);
+            let candidate = path.join(&exec);
             let with_exe = if env::consts::EXE_EXTENSION == "" {
                 None
             } else {
@@ -142,6 +142,27 @@ pub fn write(path: &Path, contents: &[u8]) -> CargoResult<()> {
     Ok(())
 }
 
+pub fn write_if_changed<P: AsRef<Path>, C: AsRef<[u8]>>(path: P, contents: C) -> CargoResult<()> {
+    (|| -> CargoResult<()> {
+        let contents = contents.as_ref();
+        let mut f = OpenOptions::new()
+            .read(true)
+            .write(true)
+            .create(true)
+            .open(&path)?;
+        let mut orig = Vec::new();
+        f.read_to_end(&mut orig)?;
+        if orig != contents {
+            f.set_len(0)?;
+            f.seek(io::SeekFrom::Start(0))?;
+            f.write_all(contents)?;
+        }
+        Ok(())
+    })()
+        .chain_err(|| format!("failed to write `{}`", path.as_ref().display()))?;
+    Ok(())
+}
+
 pub fn append(path: &Path, contents: &[u8]) -> CargoResult<()> {
     (|| -> CargoResult<()> {
         let mut f = OpenOptions::new()
